
/* GravityWars 1.1, SDL wrapper (C) 2003 Sam Hocevar <sam@zoy.org> */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "memory.h"

#ifdef USE_SDL
#include "vgastubs.h"

#include <SDL.h>

/* VGA */

#define WIDTH 640
#define HEIGHT 480

SDL_Surface *screen;
SDL_Surface *bg;
SDL_Rect score_rect, level_rect, game_rect;

int vga_init(void) {
  if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE) < 0)
    return -1;

  atexit(SDL_Quit);

  return 0;
}

int vga_setmode(int mode) {

  switch(mode) {
    case G640x480x256:
      screen = SDL_SetVideoMode(WIDTH, HEIGHT, 8,
                                SDL_HWPALETTE/*|SDL_FULLSCREEN*/);
      if(!screen) {
        fprintf(stderr, "error setting %dx%d 8bpp indexed mode: %s\n",
                WIDTH, HEIGHT, SDL_GetError());
        return -1;
      }
      break;
    case TEXT:
      break;
  }

  /* The level area in the background bitmap */
  level_rect.w = WIDTH;
  level_rect.h = HEIGHT;
  level_rect.x = 0;
  level_rect.y = 16; /* gets modified, between 16 and 960 */

  /* Where our viewport gets blitted */
  game_rect.w = WIDTH;
  game_rect.h = HEIGHT;
  game_rect.x = 0;
  game_rect.y = 0;

  /* The score area in the background bitmap */
  score_rect.w = WIDTH;
  score_rect.h = 0;
  score_rect.x = 0;
  score_rect.y = 16;

  /* Where the score gets blitted - heh, we use NULL! */

  return 0;
}

/* This one is SDL-only */
void vga_setsplitline(int sl) {
  score_rect.h = 480 - sl;
  score_rect.y = 16 - (480 - sl);
}

#ifndef MIN
#define MIN(a, b) ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a, b) ((a) > (b) ? (a) : (b))
#endif

int vga_claimvideomemory(int m) {
  bg = SDL_CreateRGBSurface(SDL_SWSURFACE, WIDTH, m / WIDTH, 8, 0, 0, 0, 0);

  if(!bg)
    return -1;

  /* set the palette to the logical screen palette so that blits
     won't be translated */
  SDL_SetColors(bg, screen->format->palette->colors, 0, 256);

  vga_setpage(0);

  return 0;
}

unsigned char *vga_getgraphmem(void) {
  return bg->pixels;
}

/* FIXME: implement this as a macro */
void vga_setpage(int page) {
  vga_ptr = bg->pixels + (page << 16);
}

void vga_waitretrace(void) {
  SDL_BlitSurface(bg, &level_rect, screen, &game_rect);
  SDL_BlitSurface(bg, &score_rect, screen, NULL);
  SDL_UpdateRect(screen, 0, 0, 0, 0);
}

/* gl */

void gl_setpalette(void *sp) {
  SDL_Color cmap[256];
  char *cols = (char *)sp;
  int i;

  for(i=0; i<256; i++) {
    cmap[i].r = *cols++ * 4;
    cmap[i].g = *cols++ * 4;
    cmap[i].b = *cols++ * 4;
  }

  SDL_SetPalette(screen, SDL_PHYSPAL, cmap, 0, 256);
}

int gl_setcontextvga(int mode) {
  return 0;
}

void  gl_setcontextvirtual(int  w,  int  h,  int  bpp, int bitspp, void *vbuf) {
  return;
}

void *gl_setdisplaystart(int x, int y) {
  level_rect.x = x;
  level_rect.y = y; /* y >= 16 */

  return NULL;
}

void gl_setpalettecolor(int c, int r, int g, int b) {
  SDL_Color cmap[1];
  cmap[0].r = r * 4;
  cmap[0].g = g * 4;
  cmap[0].b = b * 4;

  SDL_SetColors(screen, cmap, 0, 1);
}

/* Mouse */

int mouse_init(char *dev, int type, int samplerate) {
  return 0;
}

int mouse_update(void) {
  return 0;
}

int mouse_getbutton(void) {
  return 0;
}

void mouse_close(void) {
  return;
}

/* Keyboard */

unsigned char keybuf[256];

int keyboard_init(void) {
  scans = keybuf;
  memset(scans, 0, 256);
  return 0;
}

void keyboard_close(void) {
  return;
}

char *keyboard_getstate(void) {
  return scans;
}

int keyboard_update(void) {
  SDL_Event e;
  int updated = 0;
  int mod;

  mod = SDL_GetModState();
  scans[SCANCODE_LEFTCONTROL] = mod & KMOD_LCTRL;
  scans[SCANCODE_RIGHTCONTROL] = mod & KMOD_RCTRL;

  while(SDL_PollEvent(&e)) {
    if(e.type == SDL_KEYDOWN) {
      switch(e.key.keysym.sym) {
        case SDLK_ESCAPE:
          scans[SCANCODE_ESCAPE] = 1;
          updated = 1;
          break;
        case SDLK_LEFT:
          scans[SCANCODE_CURSORBLOCKLEFT] = 1;
          updated = 1;
          break;
        case SDLK_RIGHT:
          scans[SCANCODE_CURSORBLOCKRIGHT] = 1;
          updated = 1;
          break;
        case SDLK_UP:
          scans[SCANCODE_CURSORBLOCKUP] = 1;
          updated = 1;
          break;
        case SDLK_f:
          SDL_WM_ToggleFullScreen(screen);
          break;
        default:
          updated = 1;
          break;
      }
    } else if(e.type == SDL_KEYUP) {
      switch(e.key.keysym.sym) {
        case SDLK_ESCAPE:
          scans[SCANCODE_ESCAPE] = 0;
          break;
        case SDLK_LEFT:
          scans[SCANCODE_CURSORBLOCKLEFT] = 0;
          break;
        case SDLK_RIGHT:
          scans[SCANCODE_CURSORBLOCKRIGHT] = 0;
          break;
        case SDLK_UP:
          scans[SCANCODE_CURSORBLOCKUP] = 0;
          break;
        default:
          break;
      }
    }
  }

  return updated;
}

#endif /* USE_SDL */
